import pydantic
import typing


class UserProfileExtensions(pydantic.BaseModel):
    privilegeOfCommentOnUserProfile: typing.Optional[int]
    customTitles: typing.Optional[typing.Tuple]


class AvatarFrame(pydantic.BaseModel):
    status: typing.Optional[int]
    ownershipStatus: typing.Any
    version: typing.Optional[int]
    resourceUrl: typing.Optional[str]
    name: typing.Optional[str]
    icon: typing.Optional[str]
    frameType: typing.Optional[int]
    frameId: typing.Optional[str]


class Author(pydantic.BaseModel):
    status: typing.Optional[int]
    isNicknameVerified: typing.Optional[bool]
    uid: typing.Optional[str]
    level: typing.Optional[int]
    followingStatus: typing.Optional[int]
    accountMembershipStatus: typing.Optional[int]
    isGlobal: typing.Optional[bool]
    membershipStatus: typing.Optional[int]
    reputation: typing.Optional[int]
    role: typing.Optional[int]
    ndcId: typing.Optional[int]
    membersCount: typing.Optional[int]
    nickname: typing.Optional[str]
    icon: typing.Optional[str]
    avatarFrame: typing.Optional[AvatarFrame]


class UserProfile(pydantic.BaseModel):
    status: typing.Optional[int]
    moodSticker: typing.Any
    itemsCount: typing.Optional[int]
    consecutiveCheckInDays: typing.Any
    uid: typing.Optional[str]
    modifiedTime: typing.Optional[str]
    followingStatus: typing.Optional[int]
    onlineStatus: typing.Optional[int]
    accountMembershipStatus: typing.Optional[int]
    isGlobal: typing.Optional[bool]
    avatarFrameId: typing.Optional[str]
    reputation: typing.Optional[int]
    postsCount: typing.Optional[int]
    avatarFrame: typing.Optional[AvatarFrame]
    membersCount: typing.Optional[int]
    nickname: typing.Optional[str]
    mediaList: typing.Any
    icon: typing.Optional[str]
    isNicknameVerified: typing.Optional[bool]
    mood: typing.Any
    level: typing.Optional[int]
    notificationSubscriptionStatus: typing.Optional[int]
    pushEnabled: typing.Optional[bool]
    membershipStatus: typing.Optional[int]
    content: typing.Any
    joinedCount: typing.Optional[int]
    role: typing.Optional[int]
    commentsCount: typing.Optional[int]
    aminoId: typing.Optional[str]
    ndcId: typing.Optional[int]
    createdTime: typing.Optional[str]
    userProfileExtensions: typing.Optional[UserProfileExtensions]
    storiesCount: typing.Optional[int]
    blogsCount: typing.Optional[int]


class DeviceInfo(pydantic.BaseModel):
    lastClientType: typing.Optional[int]


class AdvancedSettings(pydantic.BaseModel):
    analyticsEnabled: typing.Optional[int]


class AccountExtensions(pydantic.BaseModel):
    adsFlags: typing.Optional[int]


class Account(pydantic.BaseModel):
    adsLevel: typing.Optional[int]
    deviceInfo: typing.Optional[DeviceInfo]
    mediaLabAdsMigrationJuly2020: typing.Optional[bool]
    avatarFrameId: typing.Optional[str]
    mediaLabAdsMigrationAugust2020: typing.Optional[bool]
    adsEnabled: typing.Optional[bool]

    username: typing.Any
    status: typing.Optional[int]
    uid: typing.Optional[str]
    modifiedTime: typing.Optional[str]
    twitterID: typing.Any
    activation: typing.Optional[int]
    phoneNumberActivation: typing.Optional[int]
    emailActivation: typing.Optional[int]
    appleID: typing.Any
    facebookID: typing.Any
    nickname: typing.Optional[str]
    mediaList: typing.Any
    googleID: typing.Any
    icon: typing.Optional[str]
    securityLevel: typing.Optional[int]
    phoneNumber: typing.Optional[str]
    membership: typing.Any
    advancedSettings: typing.Optional[AdvancedSettings]
    role: typing.Optional[int]
    aminoIdEditable: typing.Optional[bool]
    aminoId: typing.Optional[str]
    createdTime: typing.Optional[str]
    extensions: typing.Optional[AccountExtensions]
    email: typing.Optional[str]


class Login(pydantic.BaseModel):
    auid: typing.Optional[str]
    account: typing.Optional[Account]
    secret: typing.Optional[str]
    apiMessage: typing.Optional[str] = pydantic.Field(alias='api:message')
    sid: typing.Optional[str]
    apiStatuscode: typing.Optional[int] = pydantic.Field(alias='api:statuscode')
    apiDuration: typing.Optional[str] = pydantic.Field(alias='api:duration')
    apiTimestamp: typing.Optional[str] = pydantic.Field(alias='api:timestamp')
    userProfile: typing.Optional[UserProfile]


class LinkInfo(pydantic.BaseModel):
    objectId: typing.Optional[str]
    targetCode: typing.Optional[int]
    ndcId: typing.Optional[int]
    fullPath: typing.Any
    shortCode: typing.Any
    objectType: typing.Optional[int]
    shareURLShortCode: typing.Optional[str]
    targetCode: typing.Optional[int]
    ndcId: typing.Optional[int]
    fullPath: typing.Optional[str]
    shareURLFullPath: typing.Optional[str]


class Agent(pydantic.BaseModel):
    status: typing.Any
    ndcId: typing.Any
    membersCount: typing.Optional[int]
    icon: typing.Any
    uid: typing.Optional[str]
    followingStatus: typing.Optional[int]
    level: typing.Optional[int]
    nickname: typing.Any
    isNicknameVerified: typing.Optional[bool]
    accountMembershipStatus: typing.Optional[int]
    membershipStatus: typing.Optional[int]
    isGlobal: typing.Optional[bool]
    reputation: typing.Optional[int]
    role: typing.Any


class Ranking(pydantic.BaseModel):
    leaderboardEnabled: typing.Optional[bool]
    rankingTable: typing.Optional[typing.Tuple]
    defaultLeaderboardType: typing.Optional[int]
    leaderboardList: typing.Optional[typing.Tuple]
    enabled: typing.Optional[bool]


class Influencer(pydantic.BaseModel):
    maxVipNumbers: typing.Optional[int]
    enabled: typing.Optional[bool]
    maxVipMonthlyFee: typing.Optional[int]
    minVipMonthlyFee: typing.Optional[int]
    lock: typing.Optional[bool]


class PublicChatPrivilege(pydantic.BaseModel):
    minLevel: typing.Optional[int]
    type: typing.Optional[int]


class PublicChat(pydantic.BaseModel):
    publicChatPrivilege: typing.Optional[PublicChatPrivilege]
    enabled: typing.Optional[bool]


class AvChat(pydantic.BaseModel):
    screeningRoomEnabled: typing.Optional[bool]
    audioEnabled: typing.Optional[bool]
    videoEnabled: typing.Optional[bool]
    audio2Enabled: typing.Optional[bool]


class ModuleChat(pydantic.BaseModel):
    enabled: typing.Optional[bool]
    spamProtectionEnabled: typing.Optional[bool]
    avChat: typing.Optional[AvChat]
    publicChat: typing.Optional[PublicChat]


class TopicCategories(pydantic.BaseModel):
    enabled: typing.Optional[bool]


class Featured(pydantic.BaseModel):
    layout: typing.Optional[int]
    publicChatRoomEnabled: typing.Optional[bool]
    memberEnabled: typing.Optional[bool]
    enabled: typing.Optional[bool]
    lockMember: typing.Optional[bool]
    postEnabled: typing.Optional[bool]


class CatalogPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class Catalog(pydantic.BaseModel):
    curationEnabled: typing.Optional[bool]
    catalogPrivilege: typing.Optional[CatalogPrivilege]
    enabled: typing.Optional[bool]


class UploadPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]
    minLevel: typing.Optional[int]


class AlbumManagePrivilege(pydantic.BaseModel):
    type: typing.Optional[int]
    minLevel: typing.Optional[int]


class SharedFolder(pydantic.BaseModel):
    enabled: typing.Optional[bool]
    albumManagePrivilege: typing.Optional[AlbumManagePrivilege]
    uploadPrivilege: typing.Optional[UploadPrivilege]


class ExternalContent(pydantic.BaseModel):
    enabled: typing.Optional[bool]


class PollPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class Poll(pydantic.BaseModel):
    pollPrivilege: typing.Optional[PollPrivilege]
    enabled: typing.Optional[bool]


class LiveModePrivilege(pydantic.BaseModel):
    minLevel: typing.Optional[int]
    type: typing.Optional[int]


class LiveMode(pydantic.BaseModel):
    liveModePrivilege: typing.Optional[LiveModePrivilege]
    enabled: typing.Optional[bool]


class QuizPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class Quiz(pydantic.BaseModel):
    enabled: typing.Optional[bool]
    quizPrivilege: typing.Optional[QuizPrivilege]


class StoryPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class Story(pydantic.BaseModel):
    enabled: typing.Optional[bool]
    storyPrivilege: typing.Optional[StoryPrivilege]


class PublicChatRoomsPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]
    minLevel: typing.Optional[int]


class PublicChatRooms(pydantic.BaseModel):
    publicChatRoomsPrivilege: typing.Optional[PublicChatRoomsPrivilege]
    enabled: typing.Optional[bool]


class CatalogEntryPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class CatalogEntry(pydantic.BaseModel):
    catalogEntryPrivilege: typing.Optional[CatalogEntryPrivilege]
    enabled: typing.Optional[bool]


class BlogPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class TipCustomOption(pydantic.BaseModel):
    value: typing.Any
    icon: typing.Optional[str]


class TipOptionList(pydantic.BaseModel):
    value: typing.Optional[int]
    icon: typing.Optional[str]


class TipInfo(pydantic.BaseModel):
    tipOptionList: typing.Optional[typing.Tuple[TipOptionList, ...]]
    tipMaxCoin: typing.Optional[int]
    tippersCount: typing.Optional[int]
    tippable: typing.Optional[bool]
    tipMinCoin: typing.Optional[int]
    tipCustomOption: typing.Optional[TipCustomOption]
    tippedCoins: typing.Optional[float]


class BlogExtensionsStyle(pydantic.BaseModel):
    backgroundColor: typing.Optional[str]


class BlogExtensions(pydantic.BaseModel):
    style: typing.Optional[BlogExtensionsStyle]
    fansOnly: typing.Optional[bool]


class Blog(pydantic.BaseModel):
    globalVotesCount: typing.Optional[int]
    globalVotedValue: typing.Optional[int]
    votedValue: typing.Optional[int]
    keywords: typing.Optional[str]
    strategyInfo: typing.Optional[str]
    mediaList: typing.Optional[typing.Tuple]
    style: typing.Optional[int]
    totalQuizPlayCount: typing.Optional[int]
    title: typing.Optional[str]
    tipInfo: typing.Optional[TipInfo]
    contentRating: typing.Optional[int]
    content: typing.Optional[str]
    needHidden: typing.Optional[bool]
    guestVotesCount: typing.Optional[int]
    type: typing.Optional[int]
    status: typing.Optional[int]
    globalCommentsCount: typing.Optional[int]
    modifiedTime: typing.Optional[str]
    widgetDisplayInterval: typing.Any
    totalPollVoteCount: typing.Optional[int]
    blogId: typing.Optional[str]
    viewCount: typing.Optional[int]
    author: typing.Optional[Author]
    extensions: typing.Optional[BlogExtensions]
    votesCount: typing.Optional[int]
    ndcId: typing.Optional[int]
    createdTime: typing.Optional[str]
    endTime: typing.Any
    commentsCount: typing.Optional[int]


class ScreeningRoomPrivilege(pydantic.BaseModel):
    minLevel: typing.Optional[int]
    type: typing.Optional[int]


class ScreeningRoom(pydantic.BaseModel):
    screeningRoomPrivilege: typing.Optional[ScreeningRoomPrivilege]
    enabled: typing.Optional[bool]


class QuestionPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class Question(pydantic.BaseModel):
    enabled: typing.Optional[bool]
    questionPrivilege: typing.Optional[QuestionPrivilege]


class WebLinkPrivilege(pydantic.BaseModel):
    type: typing.Optional[int]


class WebLink(pydantic.BaseModel):
    webLinkPrivilege: typing.Optional[WebLinkPrivilege]
    enabled: typing.Optional[bool]


class Privilege(pydantic.BaseModel):
    type: typing.Optional[int]


class Image(pydantic.BaseModel):
    enabled: typing.Optional[bool]
    privilege: typing.Optional[Privilege]


class PostType(pydantic.BaseModel):
    image: typing.Optional[Image]
    webLink: typing.Optional[WebLink]
    question: typing.Optional[Question]
    screeningRoom: typing.Optional[ScreeningRoom]
    blog: typing.Optional[Blog]
    catalogEntry: typing.Optional[CatalogEntry]
    publicChatRooms: typing.Optional[PublicChatRooms]
    story: typing.Optional[Story]
    quiz: typing.Optional[Quiz]
    liveMode: typing.Optional[LiveMode]
    poll: typing.Optional[Poll]


class Post(pydantic.BaseModel):
    postType: typing.Optional[PostType]
    enabled: typing.Optional[bool]


class Module(pydantic.BaseModel):
    post: typing.Optional[Post]
    externalContent: typing.Optional[ExternalContent]
    sharedFolder: typing.Optional[SharedFolder]
    catalog: typing.Optional[Catalog]
    featured: typing.Optional[Featured]
    topicCategories: typing.Optional[TopicCategories]
    chat: typing.Optional[ModuleChat]
    influencer: typing.Optional[Influencer]
    ranking: typing.Optional[Ranking]


class CustomList(pydantic.BaseModel):
    url: typing.Optional[str]
    alias: typing.Optional[str]
    id: typing.Optional[str]


class Page(pydantic.BaseModel):
    customList: typing.Optional[typing.Tuple[CustomList, ...]]
    defaultList: typing.Optional[typing.Tuple]


class Level1(pydantic.BaseModel):
    id: typing.Optional[str]


class Level2(pydantic.BaseModel):
    id: typing.Optional[str]


class LeftSidePanelNavigation(pydantic.BaseModel):
    level2: typing.Optional[typing.Tuple[Level2, ...]]
    level1: typing.Optional[typing.Tuple[Level1, ...]]


class Style(pydantic.BaseModel):
    iconColor: typing.Any


class LeftSidePanel(pydantic.BaseModel):
    style: typing.Optional[Style]
    leftSidePanelNavigation: typing.Optional[LeftSidePanelNavigation]


class HomePage(pydantic.BaseModel):
    navigation: typing.Optional[typing.Tuple]


class Appearance(pydantic.BaseModel):
    homePage: typing.Optional[HomePage]
    leftSidePanel: typing.Optional[LeftSidePanel]


class WelcomeMessage(pydantic.BaseModel):
    text: typing.Any
    enabled: typing.Any


class General(pydantic.BaseModel):
    premiumFeatureEnabled: typing.Optional[bool]
    accountMembershipEnabled: typing.Optional[bool]
    disableLiveLayerVisible: typing.Optional[bool]
    onlyAllowOfficialTag: typing.Optional[bool]
    welcomeMessage: typing.Optional[WelcomeMessage]
    facebookAppIdList: typing.Any
    videoUploadPolicy: typing.Optional[int]
    invitePermission: typing.Optional[int]
    disableLiveLayerActive: typing.Optional[bool]
    joinedTopicIdList: typing.Optional[typing.Tuple]
    joinedBaselineCollectionIdList: typing.Optional[typing.Tuple]
    disableLocation: typing.Optional[bool]
    hasPendingReviewRequest: typing.Optional[bool]


class Configuration(pydantic.BaseModel):
    general: typing.Optional[General]
    appearance: typing.Optional[Appearance]
    page: typing.Optional[Page]
    module: typing.Optional[Module]


class ThemePack(pydantic.BaseModel):
    themePackRevision: typing.Optional[int]
    themePackUrl: typing.Optional[str]
    themeColor: typing.Optional[str]
    themePackHash: typing.Optional[str]


class CommunityAdvancedSettings(pydantic.BaseModel):
    rankingTable: typing.Optional[typing.Tuple]
    leaderboardStyle: typing.Optional[typing.Dict]
    catalogEnabled: typing.Optional[bool]
    joinedBaselineCollectionIdList: typing.Optional[typing.Tuple]
    defaultRankingTypeInLeaderboard: typing.Optional[int]
    newsfeedPages: typing.Optional[typing.Tuple]
    hasPendingReviewRequest: typing.Optional[bool]
    welcomeMessageEnabled: typing.Any
    facebookAppIdList: typing.Any
    frontPageLayout: typing.Optional[int]
    pollMinFullBarVoteCount: typing.Optional[int]
    welcomeMessageText: typing.Any


class Community(pydantic.BaseModel):
    probationStatus: typing.Optional[int]
    keywords: typing.Any
    themePack: typing.Optional[ThemePack]
    listedStatus: typing.Optional[int]
    endpoint: typing.Optional[str]
    userAddedTopicList: typing.Any
    templateId: typing.Optional[int]
    joinType: typing.Optional[int]
    searchable: typing.Optional[bool]
    advancedSettings: typing.Optional[CommunityAdvancedSettings]
    promotionalMediaList: typing.Any
    ndcId: typing.Optional[int]
    membersCount: typing.Optional[int]
    activeInfo: typing.Optional[typing.Dict]
    configuration: typing.Optional[Configuration]
    agent: typing.Optional[Agent]
    extensions: typing.Any
    createdTime: typing.Optional[str]
    mediaList: typing.Any
    name: typing.Optional[str]
    primaryLanguage: typing.Optional[str]
    isStandaloneAppMonetizationEnabled: typing.Optional[bool]
    icon: typing.Optional[str]
    isStandaloneAppDeprecated: typing.Optional[bool]
    tagline: typing.Optional[str]
    content: typing.Any
    communityHeat: typing.Optional[int]
    link: typing.Optional[str]
    modifiedTime: typing.Optional[str]
    status: typing.Optional[int]


class LinkInfoExtensions(pydantic.BaseModel):
    community: typing.Optional[Community]
    linkInfo: typing.Optional[LinkInfo]


class LinkInfoV2(pydantic.BaseModel):
    path: typing.Optional[str]
    extensions: typing.Optional[LinkInfoExtensions]


class BaseLinkInfo(pydantic.BaseModel):
    apiStatuscode: typing.Optional[int] = pydantic.Field(alias='api:statuscode')
    apiDuration: typing.Optional[str] = pydantic.Field(alias='api:duration')
    apiMessage: typing.Optional[str] = pydantic.Field(alias='api:message')
    linkInfoV2: typing.Optional[LinkInfoV2]
    apiTimestamp: typing.Optional[str] = pydantic.Field(alias='api:timestamp')


class ChatExtensions(pydantic.BaseModel):
    viewOnly: typing.Optional[bool]
    coHost: typing.Optional[typing.Tuple[str, ...]]
    membersCanInvite: typing.Optional[bool]
    language: typing.Optional[str]
    bm: typing.Optional[typing.Tuple]
    lastMembersSummaryUpdateTime: typing.Optional[int]
    fansOnly: typing.Optional[bool]
    channelType: typing.Optional[int]


class LastMessageSummary(pydantic.BaseModel):
    uid: typing.Optional[str]
    type: typing.Optional[int]
    mediaType: typing.Optional[int]
    content: typing.Optional[str]
    messageId: typing.Optional[str]
    createdTime: typing.Optional[str]
    isHidden: typing.Optional[bool]
    mediaValue: typing.Any


class MembersSummary(pydantic.BaseModel):
    status: typing.Optional[int]
    uid: typing.Optional[str]
    membershipStatus: typing.Optional[int]
    role: typing.Optional[int]
    nickname: typing.Optional[str]
    icon: typing.Optional[str]


class Chat(pydantic.BaseModel):
    userAddedTopicList: typing.Optional[typing.Tuple]
    uid: typing.Optional[str]
    membersQuota: typing.Optional[int]
    membersSummary: typing.Optional[typing.Tuple[MembersSummary, ...]]
    threadId: typing.Optional[str]
    keywords: typing.Any
    membersCount: typing.Optional[int]
    strategyInfo: typing.Optional[str]
    isPinned: typing.Optional[bool]
    title: typing.Optional[str]
    membershipStatus: typing.Optional[int]
    content: typing.Optional[str]
    needHidden: typing.Optional[bool]
    alertOption: typing.Optional[int]
    lastReadTime: typing.Optional[str]
    type: typing.Optional[int]
    status: typing.Optional[int]
    publishToGlobal: typing.Optional[int]
    modifiedTime: typing.Any
    lastMessageSummary: typing.Optional[LastMessageSummary]
    condition: typing.Optional[int]
    icon: typing.Optional[str]
    latestActivityTime: typing.Optional[str]
    author: typing.Optional[Author]
    extensions: typing.Optional[ChatExtensions]
    ndcId: typing.Optional[int]
    createdTime: typing.Any


class Message(pydantic.BaseModel):
    author: typing.Optional[Author]
    mediaValue: typing.Optional[str]
    threadId: typing.Optional[str]
    mediaType: typing.Optional[int]
    clientRefId: typing.Optional[int]
    messageId: typing.Optional[str]
    uid: typing.Optional[str]
    createdTime: typing.Optional[str]
    type: typing.Optional[int]
    isHidden: typing.Optional[bool]
    includedInSummary: typing.Optional[bool]
    chatBubbleId: typing.Optional[str]
    chatBubbleVersion: typing.Optional[int]
    extensions: typing.Optional[typing.Dict]
    ndcId: typing.Optional[int]
    content: typing.Optional[str]


class Paging(pydantic.BaseModel):
    nextPageToken: typing.Optional[str]
    prevPageToken: typing.Optional[str]


class Messages(pydantic.BaseModel):
    messageList: typing.Optional[typing.Tuple[Chat, ...]]
    paging: typing.Optional[Paging]
    apiMessage: typing.Optional[str] = pydantic.Field(alias='api:message')
    apiStatuscode: typing.Optional[int] = pydantic.Field(alias='api:statuscode')
    apiDuration: typing.Optional[str] = pydantic.Field(alias='api:duration')
    apiTimestamp: typing.Optional[str] = pydantic.Field(alias='api:timestamp')


class WalletInfo(pydantic.BaseModel):
    adsEnabled: typing.Optional[bool]
    adsFlags: typing.Optional[int]
    adsVideoStats: typing.Optional[int]
    businessCoinsEnabled: typing.Optional[bool]
    totalBusinessCoins: typing.Optional[int]
    totalBusinessCoinsFloat: typing.Optional[float]
    totalCoins: typing.Optional[int]
    totalCoinsFloat: typing.Optional[float]



from edamino import objects, api
from ujson import dumps, loads
from aiohttp import (
    ClientSession,
    ClientWebSocketResponse,
    WSServerHandshakeError
)
from typing import (
    Optional,
    Dict,
    Tuple,
    List,
    Literal, Any
)
from time import time, timezone
from base64 import b64encode
from hashlib import sha1
from hmac import new
from binascii import hexlify
from os import urandom
from uuid import UUID


def get_timestamp() -> int:
    return int(time() * 1000)


class Client:
    __slots__ = (
        'ndc_id',
        'session',
        'headers'
    )

    ndc_id: str

    session: ClientSession
    headers: Dict[str, str]

    @property
    def sid(self) -> str:
        return self.headers["NDCAUTH"][4:]

    @sid.setter
    def sid(self, sid: str) -> None:
        self.headers["NDCAUTH"] = f"sid={sid}"

    @property
    def uid(self) -> str:
        return self.headers["AUID"]

    @uid.setter
    def uid(self, uid: str) -> None:
        self.headers["AUID"] = uid

    @property
    def device_id(self) -> str:
        return self.headers["NDCDEVICEID"]

    @device_id.setter
    def device_id(self, device_id: str) -> None:
        self.headers["NDCDEVICEID"] = device_id

    def __init__(self,
                 com_id: int = 0,
                 device_id: Optional[str] = None,
                 session: Optional[ClientSession] = None) -> None:

        self.set_ndc(com_id)
        self.headers = {
            "Accept-Language": "en-US",
            "Content-Type": api.ContentType.APPLICATION_JSON,
            "User-Agent": "Dalvik/2.1.0 (Linux; U; Android 11; Redmi Note 4 Build/RQ3A.211001.001; com.narvii.amino.master/3.4.33598)",
            "Host": "service.narvii.com",
            "Accept-Encoding": "gzip",
            "Connection": "Keep-Alive",
            "NDCDEVICEID": device_id if device_id is not None else api.DEVICE_ID
        }
        self.session = session if session is not None else ClientSession(json_serialize=dumps)

    async def __aexit__(self, *args) -> None:
        await self.session.close()

    async def __aenter__(self) -> 'Client':
        return self

    def login_sid(self, sid: str, uid: str):
        self.sid = sid
        self.uid = uid

    @staticmethod
    def gen_sig(data):
        signature = b64encode(
            bytes.fromhex("32") +
            new(
                bytes.fromhex("FBF98EB3A07A9042EE5593B10CE9F3286A69D4E2"),
                data.encode("utf-8"),
                sha1
            ).digest()
        ).decode("utf-8")

        return signature

    def set_ndc(self, com_id: int) -> None:
        if com_id != 0:
            self.ndc_id = f"x{com_id}"
        else:
            self.ndc_id = "g"

    async def request(self,
                      method: Literal['POST', 'GET', 'DELETE', 'PUT'],
                      url: str,
                      json: Optional[Dict] = None,
                      full_url: bool = False) -> Dict:
        """
        Sending requests in amino.
        """
        data: Optional[str] = None
        if not full_url:
            url = f"https://service.narvii.com/api/v1/{self.ndc_id}/s/{url}"
        if json is not None:
            json['timestamp'] = get_timestamp()
            data = dumps(json)
            self.headers['NDC-MSG-SIG'] = self.gen_sig(data)

        async with self.session.request(method=method, url=url, headers=self.headers, data=data) as resp:
            response: str = await resp.text()
        if resp.status != 200:
            raise api.InvalidRequest(response)

        return loads(response)

    async def login(self, email: str, password: str) -> objects.Login:
        data = {
            "email": email,
            "v": 2,
            "secret": f"0 {password}",
            "deviceID": self.device_id,
            "clientType": 100,
            "action": "normal"
        }

        login = objects.Login(**await self.request('POST', 'auth/login', json=data))
        self.sid = login.sid
        self.uid = login.auid
        return login

    async def get_my_communities(self, start: int = 0, size: int = 25) -> Tuple[objects.Community, ...]:
        response = await self.request('GET', f'community/joined?v=1&start={start}&size={size}')
        return tuple(map(lambda community: objects.Community(**community), response['communityList']))

    async def get_info_link(self, link: str) -> objects.LinkInfoExtensions:
        base = objects.BaseLinkInfo(**await self.request('GET', f'link-resolution?q={link}'))
        return base.linkInfoV2.extensions

    async def get_user_info(self, user_id: str) -> objects.UserProfile:
        response = await self.request('GET', f'user-profile/{user_id}')
        return objects.UserProfile(**response['userProfile'])

    async def get_link_identify(self, code: str) -> Dict:
        return await self.request('GET', f'community/link-identify?q=http%3A%2F%2Faminoapps.com%2Finvite%2F{code}')

    async def join_community(self, invitation_code: Optional[str] = None) -> Dict:
        data = {}
        if invitation_code is not None:
            data["invitationId"] = await self.get_link_identify(invitation_code)
        return await self.request('POST', 'community/join', data)

    async def leave_community(self):
        return await self.request('POST', 'community/leave')

    async def upload_media(self, data: bytes, content_type: str) -> str:
        headers = {
            'NDCDEVICEID': self.device_id,
            'NDCAUTH': f"sid={self.sid}",
            'Content-Type': content_type
        }

        async with self.session.post(f"https://service.narvii.com/api/v1/g/s/media/upload",
                                     headers=headers,
                                     data=data) as response:
            text = await response.text()

        if response.status != 200:
            raise api.InvalidRequest(text)

        return loads(text)['mediaValue']

    async def download_from_link(self, link: str) -> bytes:
        async with self.session.get(link) as response:
            f = await response.read()

        if response.status != 200:
            raise api.InvalidRequest("Unable to upload file")

        return f

    async def send_image(self, image: bytes, chat_id: str) -> Dict:
        data = {
            "content": None,
            "mediaType": api.MediaType.GIF_AND_IMAGE,
            "mediaUploadValueContentType": "image/jpg",
            "mediaUhqEnabled": True,
            "mediaUploadValue": b64encode(image).decode()
        }
        return await self.request("POST", f"chat/thread/{chat_id}/message", json=data)

    async def send_audio(self, audio: bytes, chat_id: str) -> Dict:
        data = {
            "content": None,
            "type": 2,
            "mediaType": api.MediaType.AUDIO,
            "mediaUploadValue": b64encode(audio).decode()
        }
        return await self.request("POST", f"chat/thread/{chat_id}/message", json=data)

    async def send_gif(self, image: bytes, chat_id: str) -> Dict:
        data = {
            "content": None,
            "mediaType": api.MediaType.GIF_AND_IMAGE,
            "mediaUploadValueContentType": "image/gif",
            "mediaUhqEnabled": True,
            "mediaUploadValue": b64encode(image).decode()
        }
        return await self.request("POST", f"chat/thread/{chat_id}/message", json=data)

    async def send_message(self,
                           chat_id: str,
                           message: str,
                           message_type: int = 0,
                           ref_id: Optional[int] = None,
                           reply: Optional[str] = None,
                           mentions: Optional[List[str]] = None,
                           embed: Optional[api.Embed] = None,
                           link_snippets_list: Optional[List[api.LinkSnippet]] = None
                           ) -> Dict:
        if ref_id is None:
            ref_id = int(time() / 10 % 1000000000)

        if mentions is not None:
            mentions = tuple(map(lambda mention: {"uid": mention}, mentions))

        if embed is not None:
            embed = embed.dict()

        if message is not None:
            message = message.replace("<$", "‎‏").replace("$>", "‬‭")

        if link_snippets_list:
            link_snippets_list = [snippet.dict() for snippet in link_snippets_list]

        data = {
            "type": message_type,
            "content": message,
            "clientRefId": ref_id,
            "attachedObject": embed,
            "extensions": {
                "mentionedArray": mentions,
                "linkSnippetList": link_snippets_list
            },
        }
        if reply is not None:
            data["replyMessageId"] = reply
        return await self.request("POST", f"chat/thread/{chat_id}/message", json=data)

    async def get_chats(self, start: int = 0, size: int = 100) -> Tuple[objects.Chat, ...]:
        response = await self.request('GET', f'chat/thread?type=joined-me&start={start}&size={size}')
        return tuple(map(lambda chat: objects.Chat(**chat), response['threadList']))

    async def ws_connect(self) -> ClientWebSocketResponse:
        timestamp = get_timestamp()
        url = f"{self.device_id}|{timestamp}"
        headers = {
            "NDCAUTH": f"sid={self.sid}",
            "NDCDEVICEID": self.device_id,
            "NDC-MSG-SIG": self.gen_sig(url)
        }
        for i in range(1, 5):
            try:
                return await self.session.ws_connect(
                    f"wss://ws{i}.narvii.com/?signbody={self.device_id}%7C{timestamp}",
                    headers=headers
                )
            except WSServerHandshakeError:
                continue

        raise api.WebSocketConnectError("Failed to connect to remote server.")

    async def get_from_id(self, object_id: str, object_type: int = 0) -> objects.LinkInfo:
        data = {
            "objectId": object_id,
            "targetCode": 1,
            "objectType": object_type
        }
        base = objects.BaseLinkInfo(**await self.request('POST', f'https://service.narvii.com/api/v1/g/s-{self.ndc_id}/link-resolution', data, True))
        return base.linkInfoV2.extensions.linkInfo

    async def get_chat_info(self, chat_id) -> objects.Chat:
        response = await self.request('GET', f'chat/thread/{chat_id}')
        return objects.Chat(**response['thread'])

    async def get_chat_messages(self, chat_id: str, size: int = 25,
                                page_token: Optional[str] = None) -> objects.Messages:
        url = f'chat/thread/{chat_id}/message?v=2&pagingType=t&size={size}'
        if page_token is not None:
            url += f"&pageToken={page_token}"
        response = await self.request('GET', url)

        return objects.Messages(**response)

    async def get_chat_messages_iter(self, chat_id: str, size: int = 100):
        ost: int = size % 100
        whole: int = size // 100
        page_token: Optional[str] = None
        for i in range(whole):
            messages = await self.get_chat_messages(chat_id, size=100, page_token=page_token)

            page_token = messages.paging.nextPageToken
            yield messages.messageList

        yield (await self.get_chat_messages(chat_id, size=ost, page_token=page_token)).messageList

    async def get_chat_users(self, chat_id: str, start: int = 0, size: int = 25) -> Tuple[objects.UserProfile, ...]:
        response = await self.request(
            'GET',
            f'chat/thread/{chat_id}/member?start={start}&size={size}&type=default&cv=1.2'
        )
        return tuple(map(lambda user: objects.UserProfile(**user), response['memberList']))

    async def get_message_info(self):
        pass

    async def get_blog_info(self, blog_id: str) -> objects.Blog:
        response = await self.request('GET', f'blog/{blog_id}')
        return objects.Blog(**response['blog'])

    async def get_wiki_info(self, wiki_id: str) -> Dict:
        response = await self.request('GET', f'item/{wiki_id}')
        return response

    async def post_blog(self):
        pass

    async def post_wiki(self):
        pass

    async def check_in(self, tz: int = -timezone // 1000) -> Dict:
        data = {"timezone": tz}
        return await self.request('POST', 'check-in', data)

    async def lottery(self, tz: int = -timezone // 1000):
        data = {
            "timezone": tz
        }
        await self.request('POST', "check-in/lottery", data)

    async def edit_profile(self,
                           nickname: Optional[str] = None,
                           content: Optional[str] = None,
                           icon: Optional[str] = None,
                           chat_request_privilege: Optional[str] = None,
                           image_list: Optional[list] = None,
                           caption_list: Optional[list] = None,
                           background_image: Optional[str] = None,
                           background_color: Optional[str] = None,
                           titles: Optional[list] = None,
                           colors: Optional[list] = None,
                           default_bubble_id: Optional[str] = None) -> Dict:
        media_list = []
        data: Dict[str, Any] = {}

        if caption_list is not None:
            for image, caption in zip(image_list, caption_list):
                media_list.append([100, image, caption])
        else:
            if image_list is not None:
                for image in image_list:
                    media_list.append([100, image, None])
        if image_list is not None or caption_list is not None:
            data["mediaList"] = media_list
        if nickname:
            data["nickname"] = nickname
        if icon:
            data["icon"] = icon
        if content:
            data["content"] = content
        if chat_request_privilege:
            data["extensions"] = {
                "privilegeOfChatInviteRequest": chat_request_privilege
            }
        if background_image:
            data["extensions"] = {
                "style": {
                    "backgroundMediaList": [[100, background_image, None, None, None]]
                }
            }
        if background_color:
            data["extensions"] = {
                "style": {
                    "backgroundColor": background_color
                }
            }
        if default_bubble_id:
            data["extensions"] = {
                "defaultBubbleId": default_bubble_id
            }
        if titles or colors:
            tlt = []
            for titles, colors in zip(titles, colors):
                tlt.append({"title": titles, "color": colors})

            data["extensions"] = {"customTitles": tlt}

        return await self.request('POST', f'user-profile/{self.uid}', data)

    async def create_chat(self):
        pass

    async def comment_blog(self,
                           blog_id: str,
                           message: Optional[str] = None,
                           sticker_id: Optional[str] = None,
                           reply: Optional[str] = None) -> Dict:
        data = {
            "content": message,
            "stickerId": sticker_id,
            "type": 0,
            "eventSource": "PostDetailView"
        }
        if reply is not None:
            data["respondTo"] = reply
        return await self.request('POST', f'blog/{blog_id}/{"comment" if self.ndc_id != "g" else "g-comment"}', data)

    async def comment_wiki(self,
                           wiki_id: str,
                           message: Optional[str] = None,
                           sticker_id: Optional[str] = None,
                           reply: Optional[str] = None) -> Dict:
        data = {
            "content": message,
            "stickerId": sticker_id,
            "type": 0,
            "eventSource": "UserProfileView"
        }
        if reply is not None:
            data["respondTo"] = reply
        return await self.request('POST', f'item/{wiki_id}/{"comment" if self.ndc_id != "g" else "g-comment"}', data)

    async def comment_profile(self,
                              uid: str,
                              message: Optional[str] = None,
                              sticker_id: Optional[str] = None,
                              reply: Optional[str] = None) -> Dict:
        data = {
            "content": message,
            "stickerId": sticker_id,
            "type": 0,
            "eventSource": "UserProfileView"
        }
        if reply is not None:
            data["respondTo"] = reply
        return await self.request('POST', f'user-profile/{uid}/{"comment" if self.ndc_id != "g" else "g-comment"}',
                                  data)

    async def send_active_object(self,
                                 opt_in_ads_flags: int = 2147483647,
                                 tz: int = -timezone // 1000,
                                 timers: Optional[Tuple[Dict[str, int], ...]] = None
                                 ) -> Dict:
        data = {
            "userActiveTimeChunkList": timers,
            "optInAdsFlags": opt_in_ads_flags,
            "timezone": tz
        }

        return await self.request('POST', 'community/stats/user-active-time', data)

    async def like_blog(self, blog_id: str) -> Dict:
        data = {
            "value": 4,
            "eventSource": "UserProfileView"
        }
        return await self.request('POST', f"blog/{blog_id}/vote?cv=1.2", json=data)

    async def like_blogs(self, blog_ids: List[str]) -> Dict:
        data = {
            "value": 4,
            "targetIdList": blog_ids
        }
        return await self.request('POST', f"feed/vote", json=data)

    async def get_online_users(self, start: int = 0, size: int = 25) -> Tuple[objects.UserProfile, ...]:
        response = await self.request(
            "GET",
            f'live-layer?topic=ndtopic:{self.ndc_id}:online-members&start={start}&size={size}'
        )
        return tuple(map(lambda user: objects.UserProfile(**user), response["userProfileList"]))

    async def get_all_users(self,
                            users_type: Literal['recent', 'banned', 'featured', 'leaders', 'curators'] = "recent",
                            start: int = 0,
                            size: int = 25) -> Tuple[objects.UserProfile, ...]:
        response = await self.request(
            'GET',
            f'user-profile?type={users_type}&start={start}&size={size}'
        )
        return tuple(map(lambda user: objects.UserProfile(**user), response['userProfileList']))

    async def activity(self):
        pass

    async def invite_to_chat(self, uids: List[str], chat_id: str) -> Dict:
        data = {
            "uids": uids
        }
        return await self.request('POST', f'chat/thread/{chat_id}/member/invite', data)

    async def send_coins(self,
                         coins: int,
                         blog_id: Optional[str] = None,
                         chat_id: Optional[str] = None,
                         object_id: Optional[str] = None,
                         transaction_id: Optional[str] = None
                         ) -> Dict:

        url: str = ""
        if transaction_id is None:
            transaction_id = str(UUID(hexlify(urandom(16)).decode('ascii')))

        data = {
            "coins": coins,
            "tippingContext": {"transactionId": transaction_id}
        }

        if blog_id is not None:
            url = f"blog/{blog_id}/tipping"

        if chat_id is not None:
            url = f"chat/thread/{chat_id}/tipping"

        if object_id is not None:
            data["objectId"] = object_id
            data["objectType"] = 2
            url = "tipping"

        return await self.request('POST', url, json=data)

    async def subscribe(self,
                        user_id: str,
                        auto_renew: bool = False,
                        transaction_id: Optional[str] = None,
                        ) -> Dict:

        if transaction_id is None:
            transaction_id = str(UUID(hexlify(urandom(16)).decode('ascii')))
        data = {
            "paymentContext": {
                "transactionId": transaction_id,
                "isAutoRenew": auto_renew
            }
        }
        return await self.request('POST', f'influencer/{user_id}/subscribe', json=data)

    async def get_wallet_info(self) -> objects.WalletInfo:
        response = await self.request('GET', 'wallet')
        return objects.WalletInfo(**response['wallet'])

    async def join_chat(self, chat_id: str) -> Dict:
        return await self.request('POST', f'chat/thread/{chat_id}/member/{self.uid}')

    async def delete_message(self,
                             chat_id: str,
                             message_id: str,
                             as_staff: bool = False,
                             reason: Optional[str] = None) -> Dict:
        data = {
            "adminOpName": 102
        }
        if as_staff and reason:
            data["adminOpNote"] = {"content": reason}

        if not as_staff:
            return await self.request('DELETE', f'chat/thread/{chat_id}/message/{message_id}')
        else:
            return await self.request('POST', f'chat/thread/{chat_id}/message/{message_id}/admin', data)

    async def follow(self, uids: List[str]) -> Dict:
        data = {"targetUidList": uids}
        return await self.request('POST', f'user-profile/{self.uid}/joined', data)

    async def unfollow(self, uid: str) -> Dict:
        return await self.request('DELETE', f'user-profile/{self.uid}/joined/{uid}')

    async def kick_from_chat(self, chat_id: str, uid: str, allow_rejoin: bool = True) -> Dict:
        return await self.request('DELETE',
                                  f'chat/thread/{chat_id}/member/{uid}?allowRejoin={1 if allow_rejoin else 0}')

    async def get_user_blogs(self, user_id: str, start: int = 0, size: int = 25) -> Tuple[objects.Blog]:
        response = await self.request('GET', f'blog?type=user&q={user_id}&start={start}&size={size}')
        return tuple(map(lambda blog: objects.Blog(**blog), response['blogList']))

    async def pin_announcement_from_chat(self, chat_id: str, announcement: str, pin_announcement: bool = True) -> Dict:
        data = {
            "extensions": {
                "announcement": announcement,
                "pinAnnouncement": pin_announcement
            }
        }
        return await self.request('POST', f'chat/thread/{chat_id}', data)

    async def edit_chat(self,
                        chat_id: str,
                        title: Optional[str] = None,
                        icon: Optional[str] = None,
                        content: Optional[str] = None,
                        announcement: Optional[str] = None,
                        keywords: List = None,
                        pin_announcement: bool = True,
                        publish_to_global: bool = False,
                        fans_only: bool = None
                        ) -> Dict:

        data: Dict[str, Any] = {}

        if title:
            data["title"] = title
        if content:
            data["content"] = content
        if icon:
            data["icon"] = icon
        if keywords:
            data["keywords"] = keywords
        if announcement:
            data["extensions"] = {
                "announcement": announcement,
                "pinAnnouncement": pin_announcement
            }
        if fans_only:
            data["extensions"] = {"fansOnly": fans_only}

        data["publishToGlobal"] = 0 if not publish_to_global else 1

        return await self.request('POST', f'chat/thread/{chat_id}', data)

    async def set_view_only_chat(self, chat_id: str, view_only: Literal['enable', 'disable']) -> Dict:
        return await self.request('POST', f'chat/thread/{chat_id}/view-only/{view_only}')

    async def set_background_chat(self, chat_id: str, background: Optional[bytes] = None):
        data = {
            "mediaType": api.MediaType.GIF_AND_IMAGE,
            "mediaUploadValue": b64encode(background).decode(),
            "mediaUploadValueContentType": api.ContentType.IMAGE_JPG
        }
        return await self.request('POST', f'thread/{chat_id}/member/{self.uid}/background', data)

    async def set_default_background_chat(self, chat_id: str, background_number: int = 3):
        data = {
            "media": [
                100,
                f"http://static.narvii.com/default-chat-room-background/{background_number}_00.png",
                None
            ]
        }
        return await self.request('POST', f'thread/{chat_id}/member/{self.uid}/background', data)

    async def leave_chat(self, chat_id: str):
        return await self.request('DELETE', f'chat/thread/{chat_id}/member/{self.uid}')




from time import time
from os import environ

from ujson import loads
from .client import Client
from .logger import logger as log
from .context import Context
from .objects import Message
from .api import MessageType, MediaType, WebSocketConnectError

from dotenv import load_dotenv
from asyncio import get_event_loop, AbstractEventLoop
from collections import namedtuple
from typing import Optional, List, Union, Tuple, Dict, Callable

load_dotenv('.env')

Handler = namedtuple('Handler', ['media_types', 'message_types', 'callback', 'description'])
HANDLERS_COMMANDS: Dict[str, Handler] = {}
HANDLERS_EVENTS: List[Handler] = []
ON_READY: Optional[Callable] = None


class Bot:
    __slots__ = ('email', 'password', 'prefix', 'loop', 'sid', 'uid', 'timestamp')

    def __init__(self, email: str, password: str, prefix: str = ""):
        self.uid = None
        self.sid = None
        self.loop = None
        self.email = email
        self.password = password
        self.prefix = prefix
        self.timestamp = None

    def check_cfg(self):
        email = environ.get('email')
        password = environ.get('password')

        if email != self.email:
            self.sid = None
            return

        if password == self.password:
            self.sid = environ.get('sid')
            self.uid = environ.get('uid')
        else:
            self.sid = None

        try:
            self.timestamp = int(environ.get('timestamp'))
            if time() - self.timestamp > 60 * 60 * 12:
                self.sid = None
        except TypeError:
            self.timestamp = int(time())
            self.sid = None

    def update_cfg(self):
        self.timestamp = int(time())
        string = f"sid={self.sid}\n" \
                 f"uid={self.uid}\n" \
                 f"email={self.email}\n" \
                 f"password={self.password}\n" \
                 f"timestamp={self.timestamp}"

        with open('.env', 'w') as file:
            file.write(string)

    @staticmethod
    def event(
            message_types: Optional[Union[List[int], Tuple[int, ...]]] = None,
            media_types: Optional[Union[List[int], Tuple[int, ...]]] = None):

        if not message_types:
            message_types = [MessageType.TEXT]
        if not media_types:
            media_types = [MediaType.TEXT]

        def register_handler(callback):
            handler = Handler(
                description=None,
                media_types=tuple(media_types),
                message_types=tuple(message_types),
                callback=callback
            )
            HANDLERS_EVENTS.append(handler)
            return callback

        return register_handler

    def command(self,
                command: str,
                description: str = "Not descrption",
                message_types: Optional[Union[List[int], Tuple[int, ...]]] = None,
                media_types: Optional[Union[List[int], Tuple[int, ...]]] = None):

        command = command.lower()

        if not message_types:
            message_types = [MessageType.TEXT]
        if not media_types:
            media_types = [MediaType.TEXT]

        def register_handler(callback):
            handler = Handler(
                description=description,
                media_types=tuple(media_types),
                message_types=tuple(message_types),
                callback=callback
            )
            HANDLERS_COMMANDS[f"{self.prefix}{command}"] = handler

            return callback

        return register_handler

    async def __call(self, client: Client) -> None:
        ws = await client.ws_connect()
        if ON_READY:
            await ON_READY()

        while True:
            try:
                if ws.closed:
                    if time() - self.timestamp > 60 * 60 * 12:
                        await client.login(self.email, self.password)
                        log.info('Login.')
                        self.update_cfg()

                    ws = await client.ws_connect()
                    log.info('Reconnected.')

                data = await ws.receive_json(loads=loads)
                if data['t'] == 1000:
                    msg = Message(**data['o']['chatMessage'], ndcId=data['o']['ndcId'])

                    client_context = Client(session=client.session, device_id=client.device_id, com_id=msg.ndcId)
                    client_context.login_sid(self.sid, self.uid)
                    context = Context(msg=msg, client=client_context)

                    if msg.author.uid != self.uid:

                        for handler in HANDLERS_EVENTS:
                            if msg.type in handler.message_types and msg.mediaType in handler.media_types:
                                self.loop.create_task(handler.callback(context))

                        if msg.content is not None:
                            command = msg.content.split(" ")[0]
                            command = command.lower()
                            handler = HANDLERS_COMMANDS[command]
                            if '-h' in msg.content:
                                await context.reply(handler.description)
                            elif msg.type in handler.media_types and msg.mediaType in handler.media_types:
                                self.loop.create_task(handler.callback(context))

            except (TypeError, KeyError, AttributeError, WebSocketConnectError):
                continue

    def start(self, loop: Optional[AbstractEventLoop] = None, device_id: Optional[str] = None) -> None:
        self.check_cfg()
        self.loop = loop if loop is not None else get_event_loop()

        client = Client(device_id=device_id)
        try:
            if self.sid is None:
                login = self.loop.run_until_complete(client.login(self.email, self.password))
                self.sid = login.sid
                self.uid = login.auid
                log.info("Update config.")
                self.update_cfg()

            client.sid = self.sid
            client.uid = self.uid
            self.loop.run_until_complete(self.__call(client))
        except KeyboardInterrupt:
            log.info("Goodbye. ^^")
        finally:
            self.loop.run_until_complete(client.session.close())

    @staticmethod
    def on_ready(t):
        global ON_READY
        ON_READY = t



from typing import Optional

from .client import Client
from .objects import Message
from .api import Embed, LinkSnippet
from typing import (
    List
)


class Context:
    __slots__ = ('msg', 'client')

    def __init__(self, msg: Message, client: Client) -> None:
        self.msg = msg
        self.client = client

    async def reply(self,
                    message: str,
                    message_type: int = 0,
                    ref_id: Optional[int] = None,
                    mentions: Optional[List[str]] = None,
                    embed: Optional[Embed] = None,
                    link_snippets_list: Optional[List[LinkSnippet]] = None):
        return await self.client.send_message(message=message,
                                              chat_id=self.msg.threadId,
                                              reply=self.msg.messageId,
                                              message_type=message_type,
                                              ref_id=ref_id,
                                              mentions=mentions,
                                              embed=embed,
                                              link_snippets_list=link_snippets_list)

    async def send_image(self, image: bytes):
        return await self.client.send_image(image, chat_id=self.msg.threadId)

    async def send_gif(self, gif: bytes):
        return await self.client.send_gif(gif, chat_id=self.msg.threadId)

    async def send_audio(self, audio: bytes):
        return await self.client.send_audio(audio, chat_id=self.msg.threadId)

    async def download_from_link(self, link: str):
        return await self.client.download_from_link(link)

    async def send(self,
                   message: str,
                   message_type: int = 0,
                   ref_id: Optional[int] = None,
                   mentions: Optional[List[str]] = None,
                   embed: Optional[Embed] = None,
                   link_snippets_list: Optional[List[LinkSnippet]] = None):
        return await self.client.send_message(message=message,
                                              chat_id=self.msg.threadId,
                                              message_type=message_type,
                                              ref_id=ref_id,
                                              mentions=mentions,
                                              embed=embed,
                                              link_snippets_list=link_snippets_list)

    async def get_user_info(self):
        return await self.client.get_user_info(self.msg.author.uid)

    async def invite(self, chat_id: str):
        return await self.client.invite_to_chat(uids=[self.msg.author.uid], chat_id=chat_id)

    async def follow(self):
        return await self.client.follow([self.msg.author.uid])

    async def unfollow(self):
        return await self.client.unfollow(self.msg.uid)

    async def delete_message(self, as_staff: bool = False, reason: Optional[str] = None):
        return self.client.delete_message(chat_id=self.msg.threadId,
                                          message_id=self.msg.messageId,
                                          reason=reason,
                                          as_staff=as_staff)

    async def kick(self, allow_rejoin: bool = True):
        return self.client.kick_from_chat(self.msg.threadId, self.msg.author.uid, allow_rejoin)

    async def join_community(self, code: Optional[str] = None):
        return await self.client.join_community(code)

    async def leave_community(self):
        return await self.client.leave_community()

    async def join_chat(self):
        return await self.client.join_chat(self.msg.threadId)

    async def leave_chat(self):
        return await self.client.leave_chat(self.msg.threadId)



from base64 import b64encode
from typing import Optional, Dict, Tuple

DEVICE_ID = "327766716D73766C776A6F767078766740676D61696C2E636F6D5DA1DCD8E8E7C6BDDEFD7128E05113FFE25F6239"


class ContentType:
    AUDIO_AAC = "audio/aac"
    IMAGE_JPG = "image/jpg"
    IMAGE_PNG = "image/png",
    APPLICATION = "application/x-www-form-urlencoded"
    APPLICATION_JSON = 'application/json; charset=utf-8'


class InvalidRequest(Exception):
    pass


class WebSocketConnectError(Exception):
    pass


class Embed:
    __slots__ = (
        'object_id',
        'object_type',
        'link',
        'title',
        'content',
        'image'
    )

    def __init__(self,
                 object_id: Optional[str] = None,
                 object_type: Optional[int] = None,
                 link: Optional[str] = None,
                 title: Optional[str] = None,
                 content: Optional[str] = None,
                 image: Optional[str] = None) -> None:
        self.object_id = object_id
        self.object_type = object_type
        self.link = link
        self.title = title
        self.content = content
        self.image = image

    def dict(self) -> Dict:
        return {
            "objectId": self.object_id,
            "objectType": self.object_type,
            "link": self.link,
            "title": self.title,
            "content": self.content,
            "mediaList": [[100, self.image, None]] if self.image is not None else None
        }


class MessageType:
    TEXT: int = 0
    STRIKE: int = 1
    VOICE: int = 2
    STICKER: int = 3
    TYPE_USER_SHARE_EXURL: int = 50
    TYPE_USER_SHARE_USER: int = 51
    VOICE_CHAT_NOT_ANSWERED: int = 52
    VOICE_CHAT_NOT_CANCELLED: int = 53
    VOICE_CHAT_NOT_DECLINED: int = 54
    VIDEO_CHAT_NOT_ANSWERED: int = 55
    VIDEO_CHAT_NOT_CANCELLED: int = 56
    VIDEO_CHAT_NOT_DECLINED: int = 57
    AVATAR_CHAT_NOT_ANSWERED: int = 58
    AVATAR_CHAT_NOT_CANCELLED: int = 59
    AVATAR_CHAT_NOT_DECLINED: int = 60
    DELETE_MESSAGE: int = 100
    GROUP_MEMBER_JOIN: int = 101
    GROUP_MEMBER_LEAVE: int = 102
    CHAT_INVITE: int = 103
    CHAT_BACKGROUND_CHANGED: int = 104
    CHAT_TITLE_CHANGED: int = 105
    CHAT_ICON_CHANGED: int = 106
    VOICE_CHAT_START: int = 107
    VIDEO_CHAT_START: int = 108
    AVATAR_CHAT_START: int = 109
    VOICE_CHAT_END: int = 110
    VIDEO_CHAT_END: int = 111
    AVATAR_CHAT_END: int = 112
    CHAT_CONTENT_CHANGED: int = 113
    SCREEN_ROOM_START: int = 114
    SCREEN_ROOM_END: int = 115
    CHAT_HOST_TRANSFERED: int = 116
    TEXT_MESSAGE_FORCE_REMOVED: int = 117
    CHAT_REMOVED_MESSAGE: int = 118
    TEXT_MESSAGE_REMOVED_BY_ADMIN: int = 119
    CHAT_TIP: int = 120
    CHAT_PIN_ANNOUNCEMENT: int = 121
    VOICE_CHAT_PERMISSION_OPEN_TO_EVERYONE: int = 122
    VOICE_CHAT_PERMISSION_INVITED_AND_REQUESTED: int = 123
    VOICE_CHAT_PERMISSION_INVITE_ONLY: int = 124
    CHAT_VIEW_ONLY_ENABLED: int = 125
    CHAT_VIEW_ONLY_DISABLED: int = 126
    CHAT_UNPIN_ANNOUNCEMENT: int = 127
    CHAT_TIPPING_ENABLED: int = 128
    CHAT_TIPPING_DISABLED: int = 129
    TIMESTAMP_MESSAGE: int = 65281
    WELCOME_MESSAGE: int = 65282
    INVITE_MESSAGE: int = 65283
    ALL: Tuple[int, ...] = (
        TEXT,
        STRIKE,
        VOICE,
        STICKER,
        TYPE_USER_SHARE_EXURL,
        TYPE_USER_SHARE_USER,
        VOICE_CHAT_NOT_ANSWERED,
        VOICE_CHAT_NOT_CANCELLED,
        VOICE_CHAT_NOT_DECLINED,
        VIDEO_CHAT_NOT_ANSWERED,
        VIDEO_CHAT_NOT_CANCELLED,
        VIDEO_CHAT_NOT_DECLINED,
        AVATAR_CHAT_NOT_ANSWERED,
        AVATAR_CHAT_NOT_CANCELLED,
        AVATAR_CHAT_NOT_DECLINED,
        DELETE_MESSAGE,
        GROUP_MEMBER_JOIN,
        GROUP_MEMBER_LEAVE,
        CHAT_INVITE,
        CHAT_BACKGROUND_CHANGED,
        CHAT_TITLE_CHANGED,
        CHAT_ICON_CHANGED,
        VOICE_CHAT_START,
        VIDEO_CHAT_START,
        AVATAR_CHAT_START,
        VOICE_CHAT_END,
        VIDEO_CHAT_END,
        AVATAR_CHAT_END,
        CHAT_CONTENT_CHANGED,
        SCREEN_ROOM_START,
        SCREEN_ROOM_END,
        CHAT_HOST_TRANSFERED,
        TEXT_MESSAGE_FORCE_REMOVED,
        CHAT_REMOVED_MESSAGE,
        TEXT_MESSAGE_REMOVED_BY_ADMIN,
        CHAT_TIP,
        CHAT_PIN_ANNOUNCEMENT,
        VOICE_CHAT_PERMISSION_OPEN_TO_EVERYONE,
        VOICE_CHAT_PERMISSION_INVITED_AND_REQUESTED,
        VOICE_CHAT_PERMISSION_INVITE_ONLY,
        CHAT_VIEW_ONLY_ENABLED,
        CHAT_VIEW_ONLY_DISABLED,
        CHAT_UNPIN_ANNOUNCEMENT,
        CHAT_TIPPING_ENABLED,
        CHAT_TIPPING_DISABLED,
        TIMESTAMP_MESSAGE,
        WELCOME_MESSAGE,
        INVITE_MESSAGE
    )


class MediaType:
    TEXT: int = 0
    AUDIO: int = 110
    GIF_AND_IMAGE: int = 100
    STICKER: int = 113
    ALL: Tuple[int, int, int, int] = (TEXT, AUDIO, GIF_AND_IMAGE, STICKER)


class File:

    @staticmethod
    def load(path: str) -> bytes:
        with open(path, 'rb') as file:
            data = file.read()
        return data


class LinkSnippet:
    __slots__ = (
        'link',
        'media_upload_value',
        'media_type',
        'media_upload_value_content_type'
    )

    def __init__(self,
                 link: str,
                 media_upload_value: bytes,
                 media_type: int = 100,
                 media_upload_value_content_type: str = "image/png"):
        self.link = link
        self.media_upload_value = b64encode(media_upload_value).decode()
        self.media_type = media_type
        self.media_upload_value_content_type = media_upload_value_content_type

    def dict(self) -> Dict:
        return {
            "link": self.link,
            "mediaType": self.media_type,
            "mediaUploadValue": self.media_upload_value,
            "mediaUploadValueContentType": self.media_upload_value_content_type
        }



from setuptools import setup, find_packages

with open('README.md', 'r') as stream:
    long_description = stream.read()

setup(
    name='ed-amino',
    version='0.6.7',
    url='https://github.com/SvytDola/edamino',
    license='MIT',
    author='SvytDola',
    description='Async library for amino.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=[
        'edamino'
    ],
    install_requires=[
        'aiohttp',
        'ujson',
        'python-dotenv',
        'pydantic'
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    packages=find_packages()
)
